/*****************************************************************************
 * Houghtool - The Software Package for efficiency measuring and visualization
 * of the HT and it's variants for line detection
 *
 * Lappeenranta University of Technology, Department of Information Technology
 * Laboratory of Information Processing
 *
 * Authors: Petri Hirvonen, Jouni Ikonen (Jouni.Ikonen@lut.fi)
 *	    Pekka Kultanen, Heikki K{lvi{inen (Heikki.Kalviainen@lut.fi)
 *
 * File:    rht_plotfuncs.c
 * Purpose: writing data points
 * Date:    Jun 1, 1993
 *****************************************************************************/

#include "ht_hough.h"
#include "rht_infmat.h"
#define LARGE 0
#define SMALL 1

int BoundingBox[]= {25,25,241,241};  /* 3.000 in x 3.000 in */

FILE *psfile;

float find_maxv(),find_minv();

/*****************************************************************************/
DrawAccumulatorSpacePS(AccuSpace,header,x_label,y_label,xmin,xmax,ymin,ymax,
                       accumax)
/*

  Make a PostScript file of the contents of the accumulator.

  Parameters :  AccuSpace - accumulator
                header - heading text string
                x_label,y_label - axis labels
                xmin,xmax,ymin,ymax - current parameter ranges
                accumax - maximum value of the accumulator

*/
InfMat *AccuSpace;
char *header,*x_label,*y_label;
double xmin,xmax,ymin,ymax;
int accumax;
{
  if (!init_PS())
    return 0;
  fprintf(psfile,"accumulator\n");
  axis_x_abs_2PS(x_label,(float)xmin,(float)xmax);
  axis_y_abs_2PS(y_label,(float)ymin,(float)ymax);
  draw_dataPS(AccuSpace,(float)xmin,(float)xmax,
                        (float)ymin,(float)ymax,accumax);
  close_PS();
}

/*****************************************************************************/
DrawLineDataPS(x,y,n,header,x_label,y_label)
/*

  Make a PostScript file of the contents of the accumulator during the detection
  process.

  Parameters :  x,y - data vectors
                n - size of the data vectors
                header - heading text string
                x_label,y_label - axis labels

*/
float x[],y[];
int n;
char *header,*x_label,*y_label;
{
  float xmin,xmax,ymin,ymax,max;

  xmin=find_minv(x,n);
  xmax=find_maxv(x,n);
  ymin=find_minv(y,n);
  ymax=find_maxv(y,n);

  BoundingBox[0]=25;
  BoundingBox[1]=25;
  BoundingBox[2]=265;
  BoundingBox[3]=200;

  if (!init_PS())
    return 0;
  headingPS(header);
  fprintf(psfile,"%s\n",header);
  axis_x_abs_2PS(x_label,xmin,xmax);
  axis_y_abs_2PS(y_label,ymin,ymax);
  draw_line_dataPS(x,y,n,xmin,xmax,ymin,ymax);
  close_PS();
}

/*****************************************************************************/
draw_dataPS(AccuSpace,min_val_x,max_val_x,min_val_y,max_val_y,accumax)
/*

  Make a PostScript file of the contents of the accumulator.

  Parameters :  AccuSpace - accumulator
                min_val_x,max_val_x,min_val_y,max_val_y - current parameter ranges
                accumax - maximum value of the accumulator

*/
InfMat *AccuSpace;
float max_val_x,min_val_x,max_val_y,min_val_y;
int accumax;
{
  InfIndex *p=AccuSpace;
  Accu *s;
  float orig_x,orig_y,x,y,x_len,y_len,scale_x,scale_y;

  if (p->NextIndex==(InfIndex *)NULL) {
    err_prnt("Impossible to draw NULL or infinite accumulator.\n");
    return;
  }

  p=p->NextIndex;

  s=p->Accu; 

  do {
    if ( (p->X_coord>min_val_x) && (p->X_coord<max_val_x) ) {
      s=p->Accu;
      do {
        if ((s->Y_coord>min_val_y) && (s->Y_coord<max_val_y)) {
	  fprintf (psfile,"%f %f\n",p->X_coord, s->Y_coord);
        }
        s=s->Up;
      } while (s);
    }
    p=p->NextIndex;
  } while (p);
}

/*****************************************************************************/
draw_line_dataPS(x,y,n,min_val_x,max_val_x,min_val_y,max_val_y)
/*

  Make a PostScript file of the contents of the accumulator during the detection
  process.

  Parameters :  x,y - data vectors
                n - size of the data vectors
                min_val_x,max_val_x,min_val_y,max_val_y - parameter ranges

*/
float x[],y[];
int n;
float max_val_x,min_val_x,max_val_y,min_val_y;
{
  float umin,umax,vmin,vmax,orig_x,orig_y,
        x_len,y_len,scale_x,scale_y;
  int i, incr;

#if 0
  orig_x=(BoundingBox[2]-BoundingBox[0])*0.1+BoundingBox[0];
  orig_y=(BoundingBox[3]-BoundingBox[1])*0.1+BoundingBox[1];

  x_len=(BoundingBox[2]-BoundingBox[0])*0.9-(BoundingBox[2]-BoundingBox[0])*0.1;
  y_len=(BoundingBox[3]-BoundingBox[1])*0.9-(BoundingBox[3]-BoundingBox[1])*0.1;
     
  scale_x=x_len/(max_val_x-min_val_x);
  scale_y=y_len/(max_val_y-min_val_y);

  set_line_indexPS(255); 
#endif
    
  incr = n/50;
  if (incr<=0) {incr = 1;}

  for(i=1;i<n; i+=incr) {
  fprintf(psfile,"%f %f %f %f\n", x[i-1], y[i-1], x[i], y[i]);
#if 0
    move_abs_2PS((x[i-1]-min_val_x)*scale_x+orig_x,
                 (y[i-1]-min_val_y)*scale_y+orig_y);
    line_abs_2PS((x[i]-min_val_x)*scale_x+orig_x,
                 (y[i]-min_val_y)*scale_y+orig_y);
#endif
  }
}

/*****************************************************************************/
headingPS(name)
/*

  Print heading string to the PostScript file.

  Parameters :  name - heading string

*/
char *name;
{
  float label_x,label_y;
}

/*****************************************************************************/
axis_x_abs_2PS(name,min_val,max_val)
/*

  Print x-axis label string and draw sticks.

  Parameters :  name - label string
                min_val,max_val - parameter ranges

*/
char *name;
float min_val, max_val;
{
  fprintf(psfile,"%-6.2f %-6.2f\n",min_val,max_val);
}

/*****************************************************************************/
axis_y_abs_2PS(name,min_val,max_val)
/*

  Print y-axis label string and draw sticks.

  Parameters :  name - label string
                min_val,max_val - parameter ranges

*/
char *name;
float min_val, max_val;
{
  fprintf(psfile,"%-6.2f %-6.2f\n",min_val,max_val);
}


/*****************************************************************************/
init_PS()
/*

  Initialize PostScript file.

  Parameters :

*/
{
  static int n=0;
  char name[20];

  n++; 
  sprintf(name,"hough%d.ptcl",n);
  if ((psfile=fopen(name,"w"))==(FILE *)NULL) {
    err_prnt("Cannot create PS-file.\n");
    return FALSE;
  }

  return TRUE;
}

/*****************************************************************************/
close_PS()
/*

  Close PostScript file.

  Parameters :

*/
{
  fclose(psfile);
}

/*****************************************************************************/
move_rel_2PS(x,y)
/*

  Move to new relative location.

  Parameters : x,y - new location

*/
float x,y;
{
  fprintf(psfile,"N\n");
  fprintf(psfile,"%6.2f %6.2f\n",x,y);
  fprintf(psfile,"MR\n");
}

/*****************************************************************************/
line_rel_2PS(x,y)
/*

  Draw line to new relative location.

  Parameters : x,y - new location

*/
float x,y;
{
  fprintf(psfile,"%6.2f %6.2f\n",x,y);
  fprintf(psfile,"LR S\n");
}

/*****************************************************************************/
set_line_indexPS(n)
/*

  Set new line index (gray level).

  Parameters : n - index

*/
int n;
{
  fprintf(psfile,"%g setgray\n",(255.0-n)/255.0);
}

/*****************************************************************************/
textPS(x,y,text,size)
/*

  Print text to PS file.

  Parameters : x,y - text location
               text - text string
               size - font size

*/
float x,y;
char *text;
int size;
{
  if (size==LARGE)
    fprintf(psfile,"largefont setfont\n");
  else
    fprintf(psfile,"smallfont setfont\n");
  fprintf(psfile,"%6.2f %6.2f M\n",x,y);
  fprintf(psfile,"(%s)\n",text);
  fprintf(psfile,"centre\n");
  fprintf(psfile,"(%s) show\n",text);
}

/*****************************************************************************/
x_axis_textPS(x,y,text,size)
/*

  Print x-axis text to PS file.

  Parameters : x,y - text location
               text - text string
               size - font size

*/
float x,y;
char *text;
int size;
{
  if (size==LARGE)
    fprintf(psfile,"largefont setfont\n");
  else
    fprintf(psfile,"smallfont setfont\n");
  fprintf(psfile,"%6.2f %6.2f M\n",x,y);
  fprintf(psfile,"(%s)\n",text);
  fprintf(psfile,"centre\n");
  fprintf(psfile,"(%s) show\n",text);
}

/*****************************************************************************/
y_axis_textPS(x,y,text,size)
/*

  Print y-axis text to PS file.

  Parameters : x,y - text location
               text - text string
               size - font size

*/
float x,y;
char *text;
int size;
{
  if (size==LARGE)
    fprintf(psfile,"largefont setfont\n");
  else
    fprintf(psfile,"smallfont setfont\n");
  fprintf(psfile,"%6.2f %6.2f\n",-x,-y);
  fprintf(psfile,"(%s)\n",text);
  fprintf(psfile,"(%s)\n",text);
  fprintf(psfile,"%6.2f %6.2f\n",x,y);
  fprintf(psfile,"centre_rot90\n");
}

/*****************************************************************************/
move_abs_2PS(x,y)
/*

  Move to new absolute location.

  Parameters : x,y - new location

*/
float x,y;
{
  fprintf(psfile,"N\n");
  fprintf(psfile,"%6.2f %6.2f\n",x,y);
  fprintf(psfile,"M\n");
}

/*****************************************************************************/
line_abs_2PS(x,y)
/*

  Draw line to new absolute location.

  Parameters : x,y - new location

*/
float x,y;
{
  fprintf(psfile,"%6.2f %6.2f\n",x,y);
  fprintf(psfile,"L S\n");
}

/*****************************************************************************/
put_pixelPS(i,j,val)
/*

  Draw a point to PS file.

  Parameters : i,j, - point location
               val - point  gray level (not used)

*/
int i,j;
int val;
{
  fprintf(psfile,"N %6.2f %6.2f P\n",
          j/256.0*(BoundingBox[2]-BoundingBox[0])+BoundingBox[0],
          (256.0-i)/256.0*(BoundingBox[3]-BoundingBox[1])+BoundingBox[1]);
}

/*****************************************************************************/
float find_minv(x,n)
/*

  Find minimum value of the vector.

  Parameters : x - vector
               n - vector size

*/
float x[];
int n;
{
  int i;
  float fmin=1.0e10;

  for(i=0; i<n; i++)
    if (x[i]<fmin) fmin=x[i];

  return fmin;
}

/*****************************************************************************/
float find_maxv(x,n)
/*

  Find maximum value of the vector.

  Parameters : x - vector
               n - vector size

*/
float x[];
int n;
{
  int i;
  float fmax= -1.0e10;

  for(i=0; i<n; i++)
    if (x[i]>fmax) fmax=x[i];

  return fmax;
}
